
/* ask_reliable_datagram_server.pde
 *  http://www.airspayce.com/mikem/arduino/RadioHead/examples.html
 *  modified by J.Gálvez 12.03.21:  (*1) changed manager to RGDatagram.
 *                                  (*2) LED added
 *                                  (*3) settings changed -> now for ESP32
 *                                  (*4) include modified code to send an email via WiFi/SMTP server.
 *                                       (example code @https://RandomNerdTutorials.com)
 *  modified by J.Gálvez 16.04.21:  (*5) add time to connect to WiFi network; exit while loop                                                               
    modified by J.Gálvez 31.05.21:  (*6) add code for mini oled display
                                        https://github.com/greiman/SSD1306Ascii/blob/master/doc/MainPage/SSD1306mainpage.h
*/

#include "RHDatagram.h"                      //(*1)
#include "RH_ASK.h"
 
#define CLIENT_ADDRESS 2
#define SERVER_ADDRESS 1                    //(*1)
const int LedPin_B = 26;                    //(*2) blue LED for receiving RF data and email sent
const int LedPin_G = 33;                    //(*4) green LED for wifi connected and prepared for RF data
const int LedPin_R = 14;                    //(*4) red LED for error
 
// Singleton instance of the radio driver
RH_ASK driver(2000, 4, 5, 0);               //(*3) ESP8266 or ESP32: do not use pin 11 or 2
 
// Class to manage message delivery and receipt, using the driver declared above
RHDatagram manager(driver, SERVER_ADDRESS); //(*1)

#include "WiFi.h"
#include "ESP32_MailClient.h"               //(*4){

// REPLACE WITH YOUR NETWORK CREDENTIALS
const char* ssid = "yourNetwork";           //*replace it
const char* password = "yourPassword";      //*replace it

// To send Email using Gmail use port 465 (SSL) and SMTP Server smtp.gmail.com
// YOU MUST ENABLE less secure app option https://myaccount.google.com/lesssecureapps?pli=1
#define emailSenderAccount    "EXAMPLE_EMAIL@gmail.com"           //*replace it
#define emailSenderPassword   "YOUR_EXAMPLE_EMAIL_PASSWORD"       //*replace it      
#define emailRecipient        "YOUR_EMAIL_RECIPIENT@EXAMPLE.com"  //*replace it    
#define smtpServer            "smtp.gmail.com"
#define smtpServerPort        465
#define emailSubject          "Help__Ayuda__Hilfe"             

// The Email Sending data object contains config and data to send
SMTPData smtpData;

// Callback function to get the Email sending status
void sendCallback(SendStatus info);             //(*4) }

#include <Wire.h>                               //(*6)
#include "SSD1306Ascii.h"                       //(*6)
#include "SSD1306AsciiWire.h"                   //(*6)
#define I2C_ADDRESS 0x3C                        //(*6)
#define RST_PIN -1                              //(*6)
SSD1306AsciiWire oled;                          //(*6)
 
void setup(){
  Serial.begin(115200);
  byte i = 0;
  pinMode (LedPin_B, OUTPUT);                   //(*2)// Define the LED pin as output
  pinMode (LedPin_G, OUTPUT);                   //(*4)// Define the LED pin as output
  pinMode (LedPin_R, OUTPUT);                   //(*4)// Define the LED pin as output

  Wire.begin();                                 //(*6)
  Wire.setClock(400000L);                       //(*6)
  oled.begin(&Adafruit128x64, I2C_ADDRESS);     //(*6)
  oled.setFont(System5x7);                      //(*6)

  if (!manager.init()){
    Serial.println("init failed");
    
    oled.clear();                              //(*6)
    oled.println("init failed");               //(*6)
    delay(500);                                //(*6)
  }

  WiFi.begin(ssid, password);                   //(*4) {
  while (WiFi.status() != WL_CONNECTED) {
    Serial.println("Connecting...");
    oled.clear();                               //(*6)
    oled.println("help! button");               //(*6)
    oled.println("Connecting...");                //(*6)
    delay(500);                                 //(*6)
    digitalWrite(LedPin_G, HIGH);               //error: cannot connect to WiFi 
    delay(500);
    digitalWrite(LedPin_G, LOW);
    delay(500);
    Serial.printf(" %i sec\n", i);               //(*5)
    if (++i > 11) {
      Serial.print("\nWiFi Connection failed!\n\n");
      oled.println("WiFi Connection failed!");    //(*6)
      delay(500);                                 //(*6)
      break;                                      //(*5)
    }
  }

  if (WiFi.status() != WL_CONNECTED) {
    digitalWrite(LedPin_R, HIGH);
    digitalWrite(LedPin_G, LOW);
    Serial.println();
    Serial.println("WiFi connection failed.");
    oled.println("WiFi connection failed.");     //(*6)
    delay(500);                                  //(*6)
  }
  else{
    digitalWrite(LedPin_R, LOW);
    digitalWrite(LedPin_G, HIGH);                 //setup without errors 
    Serial.println();
    Serial.println("WiFi connected.");
    oled.clear();                                 //(*6)
    oled.println("help! button");                 //(*6)
    oled.println("WiFi connected.");                //(*6)
    delay(500);                                   //(*6)
    Serial.println();
    Serial.println("Prepared for sending emails");
    oled.println("* Prepared for");               //(*6)
    oled.print("sending emails*");                //(*6)
    delay(500);                                   //(*6)
    Serial.println();                             //(*4) }
  }
  
}

uint8_t buf[10];                                //(*3) Message length
 
void loop(){
  if (manager.available()){
    Serial.println("RF manager available");
    uint8_t len = sizeof(buf);                   // Wait for a message addressed to us from the client
    uint8_t from;
    if (manager.recvfrom(buf, &len, &from)){     //(*1) 
      Serial.print("got request from : 0x");
      Serial.print(from, HEX);
      Serial.print(": ");
      Serial.println((char*)buf);
      oled.clear();                             //(*6)
      oled.println("help! button");             //(*6)
      oled.println("help call from");           //(*6)
      oled.print("MARIA");                      //(*6)
      delay(500);                               //(*6)
      digitalWrite(LedPin_B, HIGH);             //(*2)
      delay(200);
      digitalWrite(LedPin_B, LOW);              //(*2)
      delay(200);
      digitalWrite(LedPin_B, HIGH);             //(*2)
      delay(200);
      digitalWrite(LedPin_B, LOW);              //(*2)
      delay(200);
      digitalWrite(LedPin_B, HIGH);             //(*2)
      delay(200);
      digitalWrite(LedPin_B, LOW);              //(*2)
      delay(200);
      digitalWrite(LedPin_R, LOW);              //(*2)
      
      // Set the SMTP Server Email host, port, account and password     //(*4) {
      smtpData.setLogin(smtpServer, smtpServerPort, emailSenderAccount, emailSenderPassword);
      // For library version 1.2.0 and later which STARTTLS protocol was supported,the STARTTLS will be 
      // enabled automatically when port 587 was used, or enable it manually using setSTARTTLS function.
      //smtpData.setSTARTTLS(true);
      // Set the sender name and Email
      smtpData.setSender("Emergency_Notification", emailSenderAccount);
      // Set Email priority or importance High, Normal, Low or 1 to 5 (1 is highest)
      smtpData.setPriority("High");
      // Set the subject
      smtpData.setSubject(emailSubject);
      // Set the message with HTML format
      smtpData.setMessage("<div style=\"color:#2f4468;\"><h1>Name of the place</h1><h3>Address:&nbsp;&nbsp; Street Number&nbsp; &nbsp;&nbsp; District&nbsp;&nbsp;&nbsp;&nbsp; City<br />reference: near the 'name of the place'</h3><p>- Restaurant 1000 m2<br />- seating capacity: 50 people<br />- ...</p><h3>Cell phone: +1 000 000 0000</h3></div>", true);
      // Set the email message in text format (raw)
      //smtpData.setMessage("Hello World! - Sent from ESP32 board", false);
      // Add recipients, you can add more than one recipient
      smtpData.addRecipient(emailRecipient);                    //*the email you added above
      //smtpData.addRecipient("YOUR_OTHER_RECIPIENT_EMAIL_ADDRESS@EXAMPLE.com");
      smtpData.setSendCallback(sendCallback);
      //Start sending Email, can be set callback function to track the status
      if (!MailClient.sendMail(smtpData)){
       Serial.println("Error sending Email, " + MailClient.smtpErrorReason());  
       digitalWrite(LedPin_B, LOW);                                     //(*4) email NOT sent 
       digitalWrite(LedPin_R, HIGH);
       }
      //Clear all data from Email object to free memory
      smtpData.empty();                                                 //(*4) }
       
    }
  }
  else {
  }
}

// Callback function to get the Email sending status                    //(*4) {
void sendCallback(SendStatus msg) {
  // Print the current status
  Serial.println(msg.info());

  // Do something when complete
  if (msg.success()) {
    Serial.println("----------------");
    oled.clear();                             //(*6)
    oled.println("help! button");             //(*6)
    oled.println("Emergency notification");   //(*6)
    oled.print("for Maria: SENT");            //(*6)
    delay(500);                               //(*6)
    digitalWrite(LedPin_B, HIGH);                                     //(*4) email sent successfully
    digitalWrite(LedPin_R, LOW);
    digitalWrite(LedPin_G, LOW);
  }
}                                                                     //(*4) }
 
